/*
 *  SupLink - URL Shortener
 *  
 *  Copyright (C) 2013  Fabien LOISON <http://www.flogisoft.com/>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package fr.flozz.supinfo.suplink.filters;

import java.io.IOException;
import java.sql.Timestamp;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import fr.flozz.supinfo.suplink.entities.Link;
import fr.flozz.supinfo.suplink.entities.User;
import fr.flozz.supinfo.suplink.entities.Visitor;
import fr.flozz.supinfo.suplink.helpers.Password;


@WebFilter(urlPatterns="/*")
public class Redirect implements Filter {
	
	private EntityManagerFactory emFactory;
	
	@Override
	public void init(FilterConfig arg0) throws ServletException {
		this.emFactory = Persistence.createEntityManagerFactory("SupLink-PU");
	}
	
	@Override
	public void destroy() {
		this.emFactory.close();
	}

	@Override
	public void doFilter(ServletRequest request_, ServletResponse response_, 
			FilterChain chain) throws IOException, ServletException {
		// Cast...
		HttpServletRequest request = (HttpServletRequest) request_;
		HttpServletResponse response = (HttpServletResponse) response_;
		// Redirection URL?
		String uriSplit[] = request.getRequestURI().split("/");
		String linkId = uriSplit[uriSplit.length-1];
		if (linkId.matches("^[0-9a-z]+$")) {
			EntityManager em = this.emFactory.createEntityManager();
			
			Query query = em.createQuery("SELECT link FROM Link link WHERE link.linkId = :linkId AND link.enabled = true");
			query.setParameter("linkId", new Long(Integer.parseInt(linkId, 36)));
			List<Link> links = query.getResultList();

			if (links.size() == 1) {
				
				Visitor visitor = new Visitor();
				visitor.setLinkId(links.get(0).getLinkId());
				visitor.setDate(new Timestamp(System.currentTimeMillis()));
				visitor.setCountry("Unknown");
				visitor.setReferer(request.getHeader("referer"));
				
				try {
					visitor.setRefererDomain(request.getHeader("referer").split("/")[0]
							+ "//" + request.getHeader("referer").split("/")[2] + "/");
				} catch (Exception e) {
					// pass
				}
				
				EntityTransaction et = em.getTransaction();
				
				try {
					et.begin();
					em.persist(visitor);
					et.commit();
				} catch (Exception e) {
					et.rollback();
				} finally {
					em.close();
				}
				
				response.sendRedirect(links.get(0).getUrl());
			} else {
				request.setAttribute("title", "SupLink - Link Error");
				request.getRequestDispatcher("/WEB-INF/nolink.jsp")
					.forward(request, response);
			}
			
		} else {
			chain.doFilter(request, response);
		}
	}
}