/*
 *  SupLink - URL Shortener
 *  
 *  Copyright (C) 2013  Fabien LOISON <http://www.flogisoft.com/>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package fr.flozz.supinfo.suplink.servlets;

import java.io.IOException;
import java.sql.Timestamp;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import fr.flozz.supinfo.suplink.entities.Link;


@WebServlet("/_add")
public class AddLink extends HttpServlet {
	
	private EntityManagerFactory emFactory;

	@Override
	public void init(ServletConfig config) throws ServletException {
		this.emFactory = Persistence.createEntityManagerFactory("SupLink-PU");
	}
	
	@Override
	public void destroy() {
		this.emFactory.close();
	}
	
	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.sendRedirect("./");
		HttpSession session = request.getSession();
		// Check
		if(!request.getParameter("url").matches("^(https?|s?ftps?|irc|news)://.+\\..+$"))
		{
			session.setAttribute("errorMsg", "The URL is not valid.");
			return;
		}
		// Fill
		Link link = new Link();
		link.setUserId((Long) session.getAttribute("userId"));
		link.setName(request.getParameter("name"));
		link.setUrl(request.getParameter("url"));
		link.setCreationDate(new Timestamp(System.currentTimeMillis()));
		link.setEnabled(true);
		// Save
		EntityManager em = this.emFactory.createEntityManager();
		EntityTransaction et = em.getTransaction();
		try {
			et.begin();
			em.persist(link);
			et.commit();
		} catch (Exception e) {
			et.rollback();
			session.setAttribute("errorMsg", "An error occured.");
		} finally {
			em.close();
			String baseUrl = request.getScheme() + "://" + request.getServerName();
			if (request.getServerPort() != 80 && request.getServerPort() != 443) {
				baseUrl += ":" + request.getServerPort();
			}
			baseUrl += request.getContextPath();
			session.setAttribute("infoMsg", "Link sucessfully shortened — "
					+baseUrl+"/"+Integer.toString(link.getLinkId().intValue(), 36));
		}
	}
}