/*
 *  SupLink - URL Shortener
 *  
 *  Copyright (C) 2013  Fabien LOISON <http://www.flogisoft.com/>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package fr.flozz.supinfo.suplink.servlets;

import java.io.IOException;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import fr.flozz.supinfo.suplink.entities.Link;


@WebServlet("/index.html")
public class Home extends HttpServlet {
	
	private EntityManagerFactory emFactory;
	
	@Override
	public void init(ServletConfig config) throws ServletException {
		this.emFactory = Persistence.createEntityManagerFactory("SupLink-PU");
	}
	
	@Override
	public void destroy() {
		this.emFactory.close();
	}
	
	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		HttpSession session = request.getSession();
		
		request.setAttribute("title", "SupLink - URL Shortener");
		
		if (session.getAttribute("userId") == null)
		{
			request.getRequestDispatcher("/WEB-INF/home_notloged.jsp")
				.forward(request, response);
		}
		else
		{
			String baseUrl = request.getScheme() + "://" + request.getServerName();
			if (request.getServerPort() != 80 && request.getServerPort() != 443) {
				baseUrl += ":" + request.getServerPort();
			}
			baseUrl += request.getContextPath();

			EntityManager em = this.emFactory.createEntityManager();
			
			Query query = em.createQuery("SELECT link FROM Link link WHERE userId = :userId ORDER BY link.linkId DESC");
			query.setParameter("userId", session.getAttribute("userId"));
			List<Link> links = query.getResultList();
			
			request.setAttribute("links", links);
			request.setAttribute("baseUrl", baseUrl);
			
			request.getRequestDispatcher("/WEB-INF/home_loged.jsp")
				.forward(request, response);
		}
	}
}