/*
 *  SupLink - URL Shortener
 *  
 *  Copyright (C) 2013  Fabien LOISON <http://www.flogisoft.com/>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package fr.flozz.supinfo.suplink.servlets;

import java.io.IOException;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import fr.flozz.supinfo.suplink.entities.User;
import fr.flozz.supinfo.suplink.helpers.Password;


@WebServlet("/_register")
public class Register extends HttpServlet {
	
	private EntityManagerFactory emFactory;
	
	@Override
	public void init(ServletConfig config) throws ServletException {
		this.emFactory = Persistence.createEntityManagerFactory("SupLink-PU");
	}
	
	@Override
	public void destroy() {
		this.emFactory.close();
	}
	
	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		HttpSession session = request.getSession();
		EntityManager em = this.emFactory.createEntityManager();
		// Redirect
				response.sendRedirect("./");
		// Check email (with a very bad regexp)
		if(!request.getParameter("regemail").matches("^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9_.+-]+\\.[a-zA-Z0-9_.+-]+$"))
		{
			session.setAttribute("errorMsg", "The email address is not valid.");
			return;
		}
		// Check password
		if(!request.getParameter("regpassword").equals(request.getParameter("regpasswordag"))
				|| request.getParameter("regpassword").isEmpty())
		{
			session.setAttribute("errorMsg", "The password fields are empty or not equal.");
			return;
		}
		// Check if user exists
		Query query = em.createQuery("SELECT user FROM User user WHERE user.email = :email");
		query.setParameter("email", request.getParameter("regemail"));
		List<User> users = query.getResultList();
		if (users.size() > 0)
		{
			session.setAttribute("errorMsg", "An user is already registered with the same email addresse.");
			return;
		}
		// Fill
		User user = new User();
		user.setEmail(request.getParameter("regemail"));
		user.setPassword(Password.hash(request.getParameter("regpassword")));
		// Save
		EntityTransaction et = em.getTransaction();
		try {
			et.begin();
			em.persist(user);
			et.commit();
		} catch (Exception e) {
			et.rollback();
			session.setAttribute("errorMsg", "An error occured.");
		} finally {
			em.close();
			session.setAttribute("infoMsg", "Your are now registered!");
		}
	}
}