/*
 *  SupLink - URL Shortener
 *  
 *  Copyright (C) 2013  Fabien LOISON <http://www.flogisoft.com/>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package fr.flozz.supinfo.suplink.servlets;

import java.io.IOException;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import fr.flozz.supinfo.suplink.entities.Link;


@WebServlet("/_toggle")
public class ToggleLink extends HttpServlet {
	
	private EntityManagerFactory emFactory;
	
	@Override
	public void init(ServletConfig config) throws ServletException {
		this.emFactory = Persistence.createEntityManagerFactory("SupLink-PU");
	}
	
	@Override
	public void destroy() {
		this.emFactory.close();
	}

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		EntityManager em = this.emFactory.createEntityManager();
		// Get Link
		Query query = em.createQuery("SELECT link FROM Link link WHERE link.linkId = :linkId");
		query.setParameter("linkId", new Long(Integer.parseInt(request.getParameter("link"), 36)));
		Link link = (Link) query.getSingleResult();
		// Toggle and save
		HttpSession session = request.getSession();
		if ((long) link.getUserId() == (long) (Long) session.getAttribute("userId")) {
			EntityTransaction et = em.getTransaction();
			link.setEnabled(!link.getEnabled());
			try {
				et.begin();
				em.persist(link);
				et.commit();
			} catch (Exception e) {
				et.rollback();
				session.setAttribute("errorMsg", "An error occured.");
			} finally {
				em.close();
			}
		}
		else
		{
			session.setAttribute("errorMsg", "You cannot enable or disable this link!");
		}
		if (request.getHeader("referer").isEmpty()) {
			response.sendRedirect("./");
		}
		else
		{
			response.sendRedirect(request.getHeader("referer"));
		}
	}
}